"use strict";

var _react = _interopRequireDefault(require("react"));

var _jestMockAxios = _interopRequireDefault(require("jest-mock-axios"));

var _testUtils = require("react-dom/test-utils");

var _useRest = _interopRequireDefault(require("../useRest"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

jest.mock('axios');

const MockComponent = () => {
  const {
    bars = [],
    post
  } = (0, _useRest.default)({
    url: '/foo',
    key: 'bar',
    pluralized: 'bars',
    runOnInit: true
  });

  const addToState = () => post({
    name: 'Bar'
  }, {});

  return /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement("ul", null, bars.map(bar => /*#__PURE__*/_react.default.createElement("li", {
    key: bar.id
  }, bar.name))), /*#__PURE__*/_react.default.createElement("button", {
    onClick: addToState,
    type: "button"
  }, "Add to state"));
};

const renderer = () => {
  const el = global.mount( /*#__PURE__*/_react.default.createElement(MockComponent, null));
  (0, _testUtils.act)(() => {
    expect(_jestMockAxios.default.get).toHaveBeenCalledWith('/foo');

    _jestMockAxios.default.mockResponse({
      data: {
        bars: [{
          id: 1,
          name: 'Quuz'
        }, {
          id: 2,
          name: 'Garply'
        }]
      }
    });
  });
  return el;
};

describe('useRest integration', () => {
  it('should get state', done => {
    const el = renderer();
    setTimeout(() => {
      el.update();
      expect(el.find('li')).toHaveLength(2);
      done();
    });
  });
  it('should add to state', done => {
    const el = renderer();
    el.find('button').simulate('click');
    (0, _testUtils.act)(() => {
      expect(_jestMockAxios.default.post).toHaveBeenCalledWith('/foo', expect.any(Object));

      _jestMockAxios.default.mockResponse({
        data: {
          bar: {
            id: 3,
            name: 'Bar'
          }
        }
      });
    });
    setTimeout(() => {
      el.update();
      expect(el.find('li')).toHaveLength(3);
      done();
    });
  });
});